/* (c) Peter Mueller, 2014 - All rights reserved. Reproduction, modification,
   use or disclosure to third parties without express
   authority is forbidden.
 */


//                MSP430F1232
//                                 -----------------
//                             /|\|              XIN|-
//                              | |                 |
//                              --|RST          XOUT|-
//                                |                 |
//                                |A10              |
//                       Ubatt >--|A0 P2.0          |
//                                |                 |
//                                |                 |
//          /|\            /|\ /|\|                 |
//           |    TMP100   11k 11k|                 |
//           |    -------   |   | |                 |
//           +---|Vcc SDA|<-|---+>|P2.3             |
//            +--|A0     |  |     |                 |
//            |--|A1     |  |     |                 |
//            |--|Vss SCL|<-+-----|P2.2             |
//           \|/  -------         |                 |


#include <msp430.h>

#include "tmp100.h"
#include "i2c.h"
#include "lib/ringbuf.h"
#include "lib/timer.h"
#include "lib/adc10.h"
#include "generated/toggle_ext.h"
#include "generated/toggle.h"
#include "generated/pwr_ext.h"
#include "generated/pwr.h"

void globalErrHandler(void);
extern void delay(unsigned int d);

// ringbuffers
FIFO_T buf1, buf2;
uint8_t buf2data[4];
uint8_t buf1data[4];


// timers
uint8_t timerId1, timerId2;

// state machines
TOGGLE_INSTANCEDATA_T smToggle = TOGGLE_INSTANCEDATA_INIT;
PWR_INSTANCEDATA_T smPwr = PWR_INSTANCEDATA_INIT;
uint16_t temperature;

/**
 * Main loop initializes the hardware and software
 * and then waits in low power mode until new events
 * are present. Events are then pulled out of the
 * queues and forwareded to the apropriate state-machine.
 */
int main( void )
{
  // Stop watchdog timer to prevent time out reset
  WDTCTL = WDTPW + WDTHOLD;
  
  // initialize timer 
  TACCTL0 |= CCIE;                           // CCR0 interrupt enabled
  TACCR0 =  410;                            
  TACTL = TASSEL_1 + MC_1 + ID_3;           // SMCLK/8, upmode  
  
  
  P1OUT &= 0x00;               // Shut down everything
  P1DIR &= 0x00;               
  P1DIR |= BIT0 + BIT1+BIT2+BIT3;       // set pins to output the rest are input 
  P1OUT |= BIT1 + BIT1+BIT2+BIT3;         //Select pull-up mode 
  
  fifoInit(&buf1, buf1data ,sizeof(buf1data)/sizeof(uint8_t));
  fifoInit(&buf2, buf2data ,sizeof(buf2data)/sizeof(uint8_t));
  
  timerInit();
  
  /*
  i2cInit();
  startTemperature();
  delay(0x8fff);
  temperature = getTemperature();
  */

  // run once to init
  toggle(&smToggle, TOGGLE_NO_MSG);
  pwr(&smPwr, PWR_NO_MSG);

  
  while(1){                      //Loop forever
     
    uint8_t retVal;
    uint8_t bufVal;
  
    do{                         // first process all events for task A
      retVal = fifoGet(&buf1, &bufVal);
      if(retVal!=QUEUE_EMPTY){
        toggle(&smToggle, bufVal);
      }
    }while(retVal!=QUEUE_EMPTY);
      
    do{                        // then all evenets for task B
      retVal = fifoGet(&buf2, &bufVal);
      if(retVal!=QUEUE_EMPTY){
        pwr(&smPwr, bufVal);
      }
    }while(retVal!=QUEUE_EMPTY);
    // more tasks could follow here
    
    __bis_SR_register(LPM3_bits + GIE);  // Enter low power mode once
    __no_operation();                    // no more events must be processed

  }
}

// Timer A0 interrupt service routine
#pragma vector=TIMER0_A0_VECTOR
__interrupt void Timer_A0(void)
{
  bool retVal;
  P1OUT ^= BIT0;        // bit0=1
  retVal = tick();

  if(retVal){
    // at least one timeout timer fired.
    // wake up main loop
    __bic_SR_register_on_exit(LPM3_bits);
  }
}


void globalErrHandler(void){
  while(1){                      //Loop forever, we do  everything with interrupts!
    __no_operation();
  }
}
